# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'memory_preferences.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFrame,
    QGridLayout, QGroupBox, QLabel, QPushButton,
    QSizePolicy, QSpacerItem, QVBoxLayout, QWidget)

from airunner.widgets.slider.slider_widget import SliderWidget

class Ui_memory_preferences(object):
    def setupUi(self, memory_preferences):
        if not memory_preferences.objectName():
            memory_preferences.setObjectName(u"memory_preferences")
        memory_preferences.resize(1007, 981)
        self.gridLayout = QGridLayout(memory_preferences)
        self.gridLayout.setObjectName(u"gridLayout")
        self.use_tiled_vae = QCheckBox(memory_preferences)
        self.use_tiled_vae.setObjectName(u"use_tiled_vae")
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        self.use_tiled_vae.setFont(font)

        self.gridLayout.addWidget(self.use_tiled_vae, 35, 0, 1, 2)

        self.line_9 = QFrame(memory_preferences)
        self.line_9.setObjectName(u"line_9")
        self.line_9.setFrameShape(QFrame.Shape.HLine)
        self.line_9.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_9, 37, 0, 1, 5)

        self.line_2 = QFrame(memory_preferences)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setFrameShape(QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_2, 25, 0, 1, 5)

        self.label_3 = QLabel(memory_preferences)
        self.label_3.setObjectName(u"label_3")
        font1 = QFont()
        font1.setPointSize(10)
        font1.setItalic(False)
        self.label_3.setFont(font1)
        self.label_3.setIndent(-1)

        self.gridLayout.addWidget(self.label_3, 20, 0, 1, 5)

        self.label_4 = QLabel(memory_preferences)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setFont(font1)
        self.label_4.setIndent(-1)

        self.gridLayout.addWidget(self.label_4, 14, 0, 1, 5)

        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.groupBox = QGroupBox(memory_preferences)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout_3 = QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.sd_combobox = QComboBox(self.groupBox)
        self.sd_combobox.setObjectName(u"sd_combobox")

        self.gridLayout_3.addWidget(self.sd_combobox, 0, 0, 1, 1)


        self.verticalLayout.addWidget(self.groupBox)

        self.groupBox_4 = QGroupBox(memory_preferences)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.gridLayout_6 = QGridLayout(self.groupBox_4)
        self.gridLayout_6.setObjectName(u"gridLayout_6")
        self.stt_combobox = QComboBox(self.groupBox_4)
        self.stt_combobox.setObjectName(u"stt_combobox")

        self.gridLayout_6.addWidget(self.stt_combobox, 0, 0, 1, 1)


        self.verticalLayout.addWidget(self.groupBox_4)

        self.groupBox_3 = QGroupBox(memory_preferences)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.gridLayout_5 = QGridLayout(self.groupBox_3)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.tts_combobox = QComboBox(self.groupBox_3)
        self.tts_combobox.setObjectName(u"tts_combobox")

        self.gridLayout_5.addWidget(self.tts_combobox, 0, 0, 1, 1)


        self.verticalLayout.addWidget(self.groupBox_3)

        self.groupBox_2 = QGroupBox(memory_preferences)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_4 = QGridLayout(self.groupBox_2)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.llm_combobox = QComboBox(self.groupBox_2)
        self.llm_combobox.setObjectName(u"llm_combobox")

        self.gridLayout_4.addWidget(self.llm_combobox, 0, 0, 1, 1)


        self.verticalLayout.addWidget(self.groupBox_2)


        self.gridLayout.addLayout(self.verticalLayout, 2, 0, 1, 5)

        self.label = QLabel(memory_preferences)
        self.label.setObjectName(u"label")
        self.label.setFont(font1)
        self.label.setIndent(-1)

        self.gridLayout.addWidget(self.label, 24, 0, 1, 5)

        self.label_5 = QLabel(memory_preferences)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font1)
        self.label_5.setIndent(-1)

        self.gridLayout.addWidget(self.label_5, 30, 0, 1, 5)

        self.enable_model_cpu_offload = QCheckBox(memory_preferences)
        self.enable_model_cpu_offload.setObjectName(u"enable_model_cpu_offload")
        self.enable_model_cpu_offload.setFont(font)

        self.gridLayout.addWidget(self.enable_model_cpu_offload, 26, 0, 1, 3)

        self.use_tf32 = QCheckBox(memory_preferences)
        self.use_tf32.setObjectName(u"use_tf32")
        self.use_tf32.setFont(font)

        self.gridLayout.addWidget(self.use_tf32, 29, 0, 1, 2)

        self.use_enable_vae_slicing = QCheckBox(memory_preferences)
        self.use_enable_vae_slicing.setObjectName(u"use_enable_vae_slicing")
        self.use_enable_vae_slicing.setFont(font)

        self.gridLayout.addWidget(self.use_enable_vae_slicing, 32, 0, 1, 2)

        self.use_tome = QGroupBox(memory_preferences)
        self.use_tome.setObjectName(u"use_tome")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(75)
        sizePolicy.setHeightForWidth(self.use_tome.sizePolicy().hasHeightForWidth())
        self.use_tome.setSizePolicy(sizePolicy)
        self.use_tome.setCheckable(True)
        self.gridLayout_2 = QGridLayout(self.use_tome)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.tome_sd_ratio = SliderWidget(self.use_tome)
        self.tome_sd_ratio.setObjectName(u"tome_sd_ratio")
        self.tome_sd_ratio.setMinimumSize(QSize(0, 20))
        self.tome_sd_ratio.setProperty("slider_callback", u"tome_sd_ratio_value_change")
        self.tome_sd_ratio.setProperty("current_value", 1000)
        self.tome_sd_ratio.setProperty("slider_maximum", 1000)
        self.tome_sd_ratio.setProperty("spinbox_maximum", 1.000000000000000)
        self.tome_sd_ratio.setProperty("display_as_float", True)
        self.tome_sd_ratio.setProperty("spinbox_single_step", 0.010000000000000)
        self.tome_sd_ratio.setProperty("spinbox_page_step", 0.100000000000000)
        self.tome_sd_ratio.setProperty("spinbox_minimum", 0.000000000000000)
        self.tome_sd_ratio.setProperty("slider_minimum", 0)

        self.gridLayout_2.addWidget(self.tome_sd_ratio, 1, 0, 1, 1)

        self.label_10 = QLabel(self.use_tome)
        self.label_10.setObjectName(u"label_10")
        font2 = QFont()
        font2.setPointSize(10)
        self.label_10.setFont(font2)

        self.gridLayout_2.addWidget(self.label_10, 0, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(624, 13, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_2.addItem(self.verticalSpacer, 2, 0, 1, 1)


        self.gridLayout.addWidget(self.use_tome, 38, 0, 1, 5)

        self.line_5 = QFrame(memory_preferences)
        self.line_5.setObjectName(u"line_5")
        self.line_5.setFrameShape(QFrame.Shape.HLine)
        self.line_5.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_5, 31, 0, 1, 5)

        self.optimize_memory_button = QPushButton(memory_preferences)
        self.optimize_memory_button.setObjectName(u"optimize_memory_button")

        self.gridLayout.addWidget(self.optimize_memory_button, 0, 0, 1, 5)

        self.line_4 = QFrame(memory_preferences)
        self.line_4.setObjectName(u"line_4")
        self.line_4.setFrameShape(QFrame.Shape.HLine)
        self.line_4.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_4, 16, 0, 1, 5)

        self.use_accelerated_transformers = QCheckBox(memory_preferences)
        self.use_accelerated_transformers.setObjectName(u"use_accelerated_transformers")
        self.use_accelerated_transformers.setFont(font)

        self.gridLayout.addWidget(self.use_accelerated_transformers, 4, 0, 1, 5)

        self.line_7 = QFrame(memory_preferences)
        self.line_7.setObjectName(u"line_7")
        self.line_7.setFrameShape(QFrame.Shape.HLine)
        self.line_7.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_7, 3, 0, 1, 1)

        self.line_8 = QFrame(memory_preferences)
        self.line_8.setObjectName(u"line_8")
        self.line_8.setFrameShape(QFrame.Shape.HLine)
        self.line_8.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_8, 10, 0, 1, 5)

        self.line_3 = QFrame(memory_preferences)
        self.line_3.setObjectName(u"line_3")
        self.line_3.setFrameShape(QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_3, 28, 0, 1, 5)

        self.use_attention_slicing = QCheckBox(memory_preferences)
        self.use_attention_slicing.setObjectName(u"use_attention_slicing")
        self.use_attention_slicing.setFont(font)

        self.gridLayout.addWidget(self.use_attention_slicing, 11, 0, 2, 2)

        self.label_8 = QLabel(memory_preferences)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setFont(font1)
        self.label_8.setIndent(-1)

        self.gridLayout.addWidget(self.label_8, 6, 0, 1, 5)

        self.use_lastchannels = QCheckBox(memory_preferences)
        self.use_lastchannels.setObjectName(u"use_lastchannels")
        self.use_lastchannels.setFont(font)

        self.gridLayout.addWidget(self.use_lastchannels, 17, 0, 2, 4)

        self.line = QFrame(memory_preferences)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line, 22, 0, 1, 5)

        self.label_11 = QLabel(memory_preferences)
        self.label_11.setObjectName(u"label_11")
        font3 = QFont()
        font3.setPointSize(12)
        font3.setBold(True)
        self.label_11.setFont(font3)

        self.gridLayout.addWidget(self.label_11, 1, 0, 1, 5)

        self.use_enable_sequential_cpu_offload = QCheckBox(memory_preferences)
        self.use_enable_sequential_cpu_offload.setObjectName(u"use_enable_sequential_cpu_offload")
        self.use_enable_sequential_cpu_offload.setFont(font)

        self.gridLayout.addWidget(self.use_enable_sequential_cpu_offload, 23, 0, 1, 5)

        self.label_2 = QLabel(memory_preferences)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setFont(font1)
        self.label_2.setIndent(-1)

        self.gridLayout.addWidget(self.label_2, 27, 0, 1, 5)

        self.label_7 = QLabel(memory_preferences)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setFont(font1)
        self.label_7.setIndent(-1)

        self.gridLayout.addWidget(self.label_7, 36, 0, 1, 2)

        self.line_6 = QFrame(memory_preferences)
        self.line_6.setObjectName(u"line_6")
        self.line_6.setFrameShape(QFrame.Shape.HLine)
        self.line_6.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout.addWidget(self.line_6, 34, 0, 1, 5)

        self.label_9 = QLabel(memory_preferences)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setFont(font2)

        self.gridLayout.addWidget(self.label_9, 8, 0, 1, 5)

        self.label_6 = QLabel(memory_preferences)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setFont(font1)
        self.label_6.setIndent(-1)

        self.gridLayout.addWidget(self.label_6, 33, 0, 1, 2)


        self.retranslateUi(memory_preferences)
        self.use_enable_vae_slicing.toggled.connect(memory_preferences.action_toggled_vae_slicing)
        self.use_tiled_vae.toggled.connect(memory_preferences.action_toggled_tile_vae)
        self.use_tf32.toggled.connect(memory_preferences.action_toggled_tf32)
        self.use_enable_sequential_cpu_offload.toggled.connect(memory_preferences.action_toggled_sequential_cpu_offload)
        self.use_accelerated_transformers.toggled.connect(memory_preferences.action_toggled_accelerated_transformers)
        self.optimize_memory_button.clicked.connect(memory_preferences.action_button_clicked_optimize_memory_settings)
        self.use_lastchannels.toggled.connect(memory_preferences.action_toggled_last_memory)
        self.use_attention_slicing.toggled.connect(memory_preferences.action_toggled_attention_slicing)
        self.enable_model_cpu_offload.toggled.connect(memory_preferences.action_toggled_model_cpu_offload)
        self.sd_combobox.currentTextChanged.connect(memory_preferences.action_changed_sd_combobox)
        self.llm_combobox.currentTextChanged.connect(memory_preferences.action_changed_llm_combobox)
        self.tts_combobox.currentTextChanged.connect(memory_preferences.action_changed_tts_combobox)
        self.stt_combobox.currentTextChanged.connect(memory_preferences.action_changed_stt_combobox)
        self.use_tome.toggled.connect(memory_preferences.action_toggled_tome)

        QMetaObject.connectSlotsByName(memory_preferences)
    # setupUi

    def retranslateUi(self, memory_preferences):
        memory_preferences.setWindowTitle(QCoreApplication.translate("memory_preferences", u"Form", None))
#if QT_CONFIG(tooltip)
        self.use_tiled_vae.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">Use with Attention Slicing or Xformers</span></p><p><span style=\" font-weight:400;\">Makes it possible to work with large images on limited VRAM. Splits image into overlapping tiles, decodes tiles, blends outputs to make final image.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.use_tiled_vae.setText(QCoreApplication.translate("memory_preferences", u"Tile vae", None))
        self.label_3.setText(QCoreApplication.translate("memory_preferences", u"May slow inference on some models, speed up on others", None))
        self.label_4.setText(QCoreApplication.translate("memory_preferences", u"Less VRAM usage, slight inference impact", None))
        self.groupBox.setTitle(QCoreApplication.translate("memory_preferences", u"Art Model", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("memory_preferences", u"Speech to text", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("memory_preferences", u"Text to speech", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("memory_preferences", u"Text Generator", None))
        self.label.setText(QCoreApplication.translate("memory_preferences", u"Less VRAM usage, slower inference", None))
        self.label_5.setText(QCoreApplication.translate("memory_preferences", u"faster matrix multiplications on ampere achitecture", None))
#if QT_CONFIG(tooltip)
        self.enable_model_cpu_offload.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">Use with attention slicing for lower memory consumption.</span></p><p><span style=\" font-weight:400;\">Moves whole models to the GPU, instead of handling each model\u2019s constituent </span><span style=\" font-weight:400; font-style:italic;\">modules</span><span style=\" font-weight:400;\">. This results in a negligible impact on inference time (compared with moving the pipeline to </span><span style=\" font-family:'monospace'; font-weight:400;\">cuda</span><span style=\" font-weight:400;\">), while still providing some memory savings.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.enable_model_cpu_offload.setText(QCoreApplication.translate("memory_preferences", u"Model CPU offload", None))
#if QT_CONFIG(tooltip)
        self.use_tf32.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">On Ampere and later CUDA devices matrix multiplications and convolutions can use the TensorFloat32 (TF32) mode for faster but slightly less accurate computations.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.use_tf32.setText(QCoreApplication.translate("memory_preferences", u"TF32", None))
#if QT_CONFIG(tooltip)
        self.use_enable_vae_slicing.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">Use with Attention Slicing or Xformers</span></p><p><span style=\" font-weight:400;\">Decode large batches of images with limited VRAM, or to enable batches with 32 images or more.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.use_enable_vae_slicing.setText(QCoreApplication.translate("memory_preferences", u"Vae Slicing", None))
#if QT_CONFIG(tooltip)
        self.use_tome.setToolTip(QCoreApplication.translate("memory_preferences", u"Merge redundant tokens for faster inference. May result in slight reduction in image quality.", None))
#endif // QT_CONFIG(tooltip)
        self.use_tome.setTitle(QCoreApplication.translate("memory_preferences", u"ToMe Token Merging", None))
        self.tome_sd_ratio.setProperty("label_text", QCoreApplication.translate("memory_preferences", u"Ratio", None))
        self.tome_sd_ratio.setProperty("settings_property", QCoreApplication.translate("memory_preferences", u"memory_settings.tome_sd_ratio", None))
        self.label_10.setText(QCoreApplication.translate("memory_preferences", u"Faster inference, slight image impact", None))
        self.optimize_memory_button.setText(QCoreApplication.translate("memory_preferences", u"Optimize Memory Settings", None))
#if QT_CONFIG(tooltip)
        self.use_accelerated_transformers.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">Optimized and memory-efficient attention implementation.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.use_accelerated_transformers.setText(QCoreApplication.translate("memory_preferences", u"Accelerated Transformers", None))
#if QT_CONFIG(tooltip)
        self.use_attention_slicing.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">Perform computation in steps instead of all at once.</span></p><p><span style=\" font-weight:400;\">About 10% slower inference times.</span></p><p><span style=\" font-weight:400;\">Uses as little as 3.2 GB of VRAM.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.use_attention_slicing.setText(QCoreApplication.translate("memory_preferences", u"Attention Slicing", None))
        self.label_8.setText(QCoreApplication.translate("memory_preferences", u"Faster inference, lower VRAM usage", None))
#if QT_CONFIG(tooltip)
        self.use_lastchannels.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">Alternative way of ordering NCHW tensors in memory preserving dimensions ordering. Channels last tensors ordered in such a way that channels become the densest dimension (aka storing images pixel-per-pixel). </span></p><p><span style=\" font-weight:400;\">Since not all operators currently support channels last format it may result in a worst performance, so it\u2019s better to try it and see if it works for your model.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.use_lastchannels.setText(QCoreApplication.translate("memory_preferences", u"Channels last memory", None))
        self.label_11.setText(QCoreApplication.translate("memory_preferences", u"Assign models to available cards", None))
#if QT_CONFIG(tooltip)
        self.use_enable_sequential_cpu_offload.setToolTip(QCoreApplication.translate("memory_preferences", u"<html><head/><body><p><span style=\" font-weight:400;\">Use with </span>attention slicing<span style=\" font-weight:400;\"> for lower memory consumption.</span></p><p><span style=\" font-weight:400;\">Offloads the weights to CPU and only load them to GPU when performing the forward pass for memory savings.</span></p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.use_enable_sequential_cpu_offload.setText(QCoreApplication.translate("memory_preferences", u"Sequential CPU offload", None))
        self.label_2.setText(QCoreApplication.translate("memory_preferences", u"Less VRAM usage, slight inference impact", None))
        self.label_7.setText(QCoreApplication.translate("memory_preferences", u"Work with large images", None))
        self.label_9.setText(QCoreApplication.translate("memory_preferences", u"Keep this checked to take advantage of torch 2.0", None))
        self.label_6.setText(QCoreApplication.translate("memory_preferences", u"Work with large batches", None))
    # retranslateUi




from setuptools import setup, find_packages

setup(
    name="airunner",
    version="3.4.0",
    author="Capsize LLC",
    description="A Stable Diffusion GUI",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords="ai, stable diffusion, art, ai art, stablediffusion",
    license="Apache-2.0",
    author_email="contact@capsizegames.com",
    url="https://github.com/Capsize-Games/airunner",
    package_dir={"": "src"},
    packages=find_packages("src"),
    python_requires=">=3.10.12,<3.11",
    install_requires=[
        "torch==2.6.0",
        "torchvision==0.21.0",
        "torchaudio==2.6.0",
        "accelerate==1.3.0",
        "huggingface-hub==0.28.1",
        "tokenizers==0.21.0",
        "optimum==1.24.0",
        "numpy==1.26.4",
        "pillow==11.1.0",
        "alembic==1.14.1",
        "aiosqlite==0.21.0",
        "sqlalchemy==2.0.38",
        "setuptools==75.8.0",
    ],
    extras_require={
        "gui": [
            "PySide6==6.7.0",
            "PySide6_Addons==6.7.0",
            "PySide6_Essentials==6.7.0",
        ],
        "linux": [
            "faiss-gpu==1.7.2",
            "tensorrt==10.9.0.34",
        ],
        "dev": [
            "pytest",
            "python-dotenv==1.0.1"
        ],
        "art": [
            "DeepCache==0.1.1",
            "diffusers==0.32.2",
            "controlnet_aux==0.0.9",
            "safetensors==0.5.2",
            "compel==2.0.3",
            "tomesd==0.1.3",
        ],
        "llm": [
            "transformers==4.48.1",
            "auto-gptq==0.7.1",
            "bitsandbytes==0.45.2",
            "datasets==3.2.0",
            "sentence_transformers==3.4.1",
            "sounddevice==0.5.1",
            "pyttsx3==2.91",
            "cryptography==44.0.0",
            "llama-index==0.12.14",
            "llama-index-readers-file==0.4.4",
            "llama-index-readers-web==0.3.5",
            "llama-index-llms-huggingface==0.4.2",
            "llama-index-llms-groq==0.3.1",
            "llama-index-embeddings-mistralai==0.3.0",
            "llama-index-vector-stores-faiss==0.3.0",
            "llama-index-embeddings-huggingface==0.5.1",
            "llama-index-llms-openrouter==0.3.1",
            "langchain-community==0.3.17",
            "EbookLib==0.18",
            "html2text==2024.2.26",
            "rake_nltk==1.0.6",
            "tf-keras==2.18.0",
            "peft==0.14.0",
            "lxml_html_clean==0.4.1",
        ],
        "llm_weather": [
            "requests-cache==1.2.1",
            "retry-requests==2.0.0",
            "openmeteo_requests==1.3.0",
        ],
        "tts": [
            "inflect==7.5.0",
            "pycountry==24.6.1",
        ]
    },
    package_data={
        "airunner": [
            "alembic/*",
            "cursors/*",
            "filters/*",
            "icons/*",
            "images/*",
            "styles/*",
            "widgets/**/*.ui",
            "windows/**/*.ui",
            "*.qrc",
            "*.ini",
        ],
    },
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'airunner=airunner.main:main',
        ],
    },
)



from airunner.handlers.llm.agent.external_condition_stopping_criteria import ExternalConditionStoppingCriteria
from airunner.handlers.llm.agent.html_file_reader import HtmlFileReader
from airunner.handlers.llm.agent.rag_mixin import RAGMixin
from airunner.handlers.llm.agent.weather_mixin import WeatherMixin


__all__ = [
    "ExternalConditionStoppingCriteria",
    "HtmlFileReader",
    "RAGMixin",
    "WeatherMixin",
]



from PySide6.QtCore import Slot

from airunner.widgets.base_widget import BaseWidget
from airunner.widgets.tool_tab.templates.tool_tab_ui import Ui_tool_tab_widget
from airunner.data.models import Tab


class ToolTabWidget(BaseWidget):
    widget_class_ = Ui_tool_tab_widget

    def __init__(self, *args, **kwargs):
        self.splitters = [
            "llm_splitter"
        ]
        super().__init__(*args, **kwargs)        
        self.ui.tool_tab_widget_container.currentChanged.connect(
            self.on_tab_section_changed
        )

    @Slot(int)
    def on_tab_section_changed(self, index: int):
        Tab.update_tabs("right", self.ui.tool_tab_widget_container, index)
    
    def restore_state(self):
        active_index = 0
        tabs = Tab.objects.filter_by(section="right")
        for tab in tabs:
            if tab.active:
                active_index = tab.index
                break
        self.ui.tool_tab_widget_container.setCurrentIndex(active_index)
        super().restore_state()



from sqlalchemy import Column, Integer, String, JSON

import datetime

from airunner.data.models.base import BaseModel


class Chatstore(BaseModel):
    __tablename__ = 'chatstore'
    id = Column(Integer, primary_key=True, autoincrement=True)
    date_created = Column(String, default=datetime.datetime.now().isoformat())
    key = Column(String, nullable=False, default="")
    value = Column(JSON, nullable=False, default={})



from dataclasses import dataclass


@dataclass
class Rect:
    x: int
    y: int
    width: int
    height: int

    def left(self) -> int:
        return self.x
    
    def top(self) -> int:
        return self.y

    def translate(self, x: int, y: int):
        self.x += x
        self.y += y


import unittest
from airunner.handlers import SpeechT5Handler
from airunner.utils import (
    replace_numbers_with_words,
    roman_to_int,
    replace_unspeakable_characters,
    strip_emoji_characters,
    prepare_text_for_tts
)


class TestTextPreprocessing(unittest.TestCase):

    def test_replace_numbers_with_words(self):
        # Test cases
        test_cases = {
            "12:00PM": "twelve P M",
            "3:34AM": "three thirty four A M",
            "12:33pm": "twelve thirty three P M",
            "34PM": "thirty four P M",
            "I have 2 apples": "I have two apples",
            "The time is 12:30": "The time is twelve thirty",
            "He is 25 years old": "He is twenty five years old",
            "Room number 404": "Room number four hundred and four",
            "No numbers here": "No numbers here"
        }

        for input_text, expected_output in test_cases.items():
            with self.subTest(input_text=input_text, expected_output=expected_output):
                self.assertEqual(replace_numbers_with_words(input_text), expected_output)

    def test_roman_to_int(self):
        # Test cases for Roman numerals
        test_cases = {
            "I": "1",
            "IV": "4",
            "IX": "9",
            "XII": "12",
            "XXI": "21",
            "XL": "40",
            "L": "50",
            "XC": "90",
            "C": "100",
            "CD": "400",
            "D": "500",
            "CM": "900",
            "M": "1000",
            "MMXXI": "2021",
            "This is a IV test": "This is a 4 test",
            "A test with no roman numerals": "A test with no roman numerals",
        }

        for roman, expected in test_cases.items():
            with self.subTest(roman=roman, expected=expected):
                self.assertEqual(roman_to_int(roman), expected)

    def test_replace_unspeakable_characters(self):
        # Test cases
        test_cases = {
            "Hello... world!": "Hello  world!",
            "This is an ellipsis…": "This is an ellipsis ",
            'Smart quotes \'single\' and "double"': "Smart quotes single and double",
            "Em dash — and en dash –": "Em dash  and en dash ",
            "Tabs\tand\nnewlines\r\n": "Tabs and newlines ",
        }

        for input_text, expected_output in test_cases.items():
            with self.subTest(input_text=input_text, expected_output=expected_output):
                self.assertEqual(replace_unspeakable_characters(input_text), expected_output)

    def test_strip_emoji_characters(self):
        # Test cases
        test_cases = {
            "😊": "",
            "😂": "",
            "👍": "",
            "🏆": "",
            "😊😂👍🏆": "",
            "😊😊😊": "",
            "😂👍🏆😊": "",
            "🏆👍😂😊": "",
            "👍🏆😊😂": "",
            "No emojis here": "No emojis here"
        }

        for input_text, expected_output in test_cases.items():
            with self.subTest(input_text=input_text, expected_output=expected_output):
                self.assertEqual(strip_emoji_characters(input_text), expected_output)

    def test_prepare_text_for_tts(self):
        # Test cases
        test_cases = {
            "Emoji 😊 should be removed": "Emoji should be removed",
            "Mixed 😊 text  with 'quotes' and — dashes": "Mixed text with quotes and dashes",
            "😊": "",
            "Hello 😊": "Hello",
            "😊😊😊": "",
            "Mixed text 😊 with emoji": "Mixed text with emoji",
            "Multiple emojis 😊😂👍": "Multiple emojis",
            "Text with various emojis 😊😂👍🏆": "Text with various emojis",
            "Emojis at the end 😊😂👍🏆": "Emojis at the end",
            "No emojis here": "No emojis here",
            "It's 25°C outside": "Its twenty five degrees Celsius outside"  # Updated to match actual behavior
        }

        for input_text, expected_output in test_cases.items():
            with self.subTest(input_text=input_text, expected_output=expected_output):
                processed = prepare_text_for_tts(input_text)
                # Account for extra spaces that might be introduced during processing
                processed = " ".join(processed.split())
                expected_output = " ".join(expected_output.split())
                self.assertEqual(processed, expected_output)


class TestSpeechT5Handler(unittest.TestCase):
    """Tests specific to the SpeechT5Handler that aren't covered by text preprocessing tests"""
    
    def test_handler_initialization(self):
        handler = SpeechT5Handler()
        self.assertIsNotNone(handler)
        # Add more handler-specific tests if needed


if __name__ == '__main__':
    unittest.main()



from transformers import StoppingCriteria


class ExternalConditionStoppingCriteria(StoppingCriteria):
    def __init__(self, external_condition_callable):
        super().__init__()
        self.external_condition_callable = external_condition_callable

    def __call__(self, inputs_ids, scores):
        return self.external_condition_callable()



from typing import Optional, List, TypeVar, Any
import logging
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.inspection import inspect
from sqlalchemy.orm import Query, joinedload

import traceback

from airunner.data.session_manager import session_scope

_T = TypeVar("_T", bound=Any)

Base = declarative_base()
logger = logging.getLogger(__name__)


class BaseManager:
    def __init__(self, cls):
        self.cls = cls

    def get(self, pk) -> Optional[_T]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).filter(self.cls.id == pk).first()
                logger.debug(f"Query result for get({pk}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in get({pk}): {e}")
                return None

    def first(
        self,
        eager_load: Optional[List[str]] = None
    ) -> Optional[_T]:
        with session_scope() as session:
            try:
                query = session.query(self.cls)
                if eager_load:
                    for relationship in eager_load:
                        try:
                            query = query.options(joinedload(getattr(
                                self.cls, 
                                relationship
                            )))
                        except AttributeError:
                            logger.warning((
                                f"Class {self.cls.__name__} does "
                                "not have relationship {relationship}"
                            ))
                            pass
                result = query.first()
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in first(): {e}")
                return None

    def all(self) -> List[_T]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).all()
                logger.debug(f"Query result for all(): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in all(): {e}")
                return []

    def filter_by(self, **kwargs) -> Optional[List[_T]]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).filter_by(**kwargs).all()
                logger.debug(f"Query result for filter_by({kwargs}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in filter_by({kwargs}): {e}")
                return None
    
    def filter_first(self, *args) -> Optional[_T]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).filter(*args).first()
                logger.debug(f"Query result for filter({args}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in filter({args}): {e}")
                return None

    def filter(self, *args) -> Optional[List[_T]]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).filter(*args).all()
                logger.debug(f"Query result for filter({args}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in filter({args}): {e}")
                return None

    def filter_by_first(self, **kwargs) -> Optional[_T]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).filter_by(**kwargs).first()
                logger.debug(f"Query result for filter_by({kwargs}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in filter_by({kwargs}): {e}")
                return None
    
    def order_by(self, *args) -> Optional[Query]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).order_by(*args)
                logger.debug(f"Query result for order_by({args}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in order_by({args}): {e}")
                return None
    
    def options(self, *args) -> Optional[Query]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).options(*args)
                logger.debug(f"Query result for options({args}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in options({args}): {e}")
                return None

    def delete_all(self) -> int:
        with session_scope() as session:
            try:
                result = session.query(self.cls).delete()
                session.commit()
                logger.debug(f"Deleted {result} {self.cls.__name__} objects")
                return result
            except Exception as e:
                logger.error(f"Error in delete(): {e}")
                return 0

    def delete_by(self, **kwargs) -> bool:
        with session_scope() as session:
            try:
                result = session.query(self.cls).filter_by(**kwargs).delete()
                session.commit()
                logger.debug(f"Deleted {result} {self.cls.__name__} objects")
                return result
            except Exception as e:
                logger.error(f"Error in delete_by({kwargs}): {e}")
                return False

    def delete(self, pk=None, **kwargs) -> bool:
        with session_scope() as session:
            try:
                if pk:
                    obj = session.query(self.cls).filter(self.cls.id == pk).first()
                    if obj:
                        session.delete(obj)
                        session.commit()
                        logger.debug(f"Deleted {self.cls.__name__} with id {pk}")
                        return True
                    else:
                        logger.debug(f"No {self.cls.__name__} found with id {pk}")
                        return False
                elif kwargs:
                    result = session.query(self.cls).filter_by(**kwargs).delete()
                    session.commit()
                    logger.debug(f"Deleted {result} {self.cls.__name__} objects")
                    return result
                else:
                    logger.debug("No arguments provided to delete()")
                    return False
            except Exception as e:
                logger.error(f"Error in delete({pk}): {e}")
                return False
    
    def update(self, pk, **kwargs) -> bool:
        with session_scope() as session:
            try:
                obj = session.query(self.cls).filter(self.cls.id == pk).first()
                if obj:
                    for key, value in kwargs.items():
                        setattr(obj, key, value)
                    session.commit()
                    logger.debug(f"Updated {self.cls.__name__} with id {pk}")
                    return True
                else:
                    logger.debug(f"No {self.cls.__name__} found with id {pk}")
                    return False
            except Exception as e:
                logger.error(f"Error in update({pk}, {kwargs}): {e}")
                return False
    
    def merge(self, obj) -> bool:
        with session_scope() as session:
            try:
                session.merge(obj)
                session.commit()
                logger.debug(f"Merged {self.cls.__name__}")
                return True
            except Exception as e:
                logger.error(f"Error in merge(): {e}")
                return False
    
    def distinct(self, *args) -> Optional[Query]:
        with session_scope() as session:
            try:
                result = session.query(self.cls).distinct(*args)
                logger.debug(f"Query result for distinct({args}): {result}")
                session.expunge_all()
                return result
            except Exception as e:
                logger.error(f"Error in distinct({args}): {e}")
                return None
    
    def create(self, *args, **kwargs) -> Optional[_T]:
        obj = None
        with session_scope() as session:
            try:
                obj = self.cls(*args, **kwargs)
                session.add(obj)
                session.commit()
                logger.debug(f"Created {self.cls.__name__}")
            except Exception as e:
                logger.error(f"Error in create({args}, {kwargs}): {e}")
            finally:
                session.expunge(obj)
        return obj


class BaseModel(Base):
    __abstract__ = True

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls.objects = BaseManager(cls)
    
    def save(self):
        with session_scope() as session:
            session.add(self)
            try:
                session.commit()
                logger.debug(f"Saved {self}")
            except Exception as e:
                logger.error(f"Error in save(): {e}")
            finally:
                session.expunge(self)
        
    def delete(self):
        success = False
        with session_scope() as session:
            try:
                session.delete(self)
                session.commit()
                logger.debug(f"Deleted {self.cls}")
                success = True
            except Exception as e:
                logger.error(f"Error in delete(): {e}")
            session.expunge(self)
        return success

    def to_dict(self):
        return {c.key: getattr(self, c.key) for c in inspect(self).mapper.column_attrs}



from PIL.ImageFilter import BoxBlur as ImageFilterBoxBlur
from airunner.filters.base_filter import BaseFilter


class BoxBlur(BaseFilter):
    def apply_filter(self, image, do_reset=False):
        return image.filter(ImageFilterBoxBlur(radius=self.radius))



import os
import json
from typing import Optional, List, Tuple
from datasets import Dataset
from transformers import TrainingArguments, Trainer


class TrainingMixin:
    @property
    def finetuned_model_directory(self) -> str:
        return os.path.expanduser(
            os.path.join(
                self.path_settings.base_path,
                "text",
                "models",
                "llm",
                "causallm",
                self.model_version,
                "fine_tuned_mistral_qllm"
            )
        )

    @property
    def latest_checkpoint(self) -> Optional[str]:
        latest_checkpoint = None
        if os.path.exists(self.finetuned_model_directory):
            checkpoints = [
                os.path.join(
                    self.finetuned_model_directory, 
                    d
                ) for d in os.listdir(
                    self.finetuned_model_directory
                ) if d.startswith(
                    "checkpoint-"
                )
            ]
            if checkpoints:
                latest_checkpoint = max(checkpoints, key=os.path.getmtime)
        return latest_checkpoint
    
    @property
    def adapter_path(self):
        base = self.path_settings.base_path
        return os.path.expanduser(os.path.join(
            base,
            "text",
            "models",
            "llm",
            "causallm",
            self.model_version,
            "user_memory_adapter"
        ))
    
    def train(
        self, 
        training_data: List[Tuple[str, str]],
        username: str = "User",
        botname: str = "Assistant",
        training_steps: int = 60,
        save_steps: int = 1,
        num_train_epochs: int = 200,
        learning_rate: float = 2e-4,
        gradient_accumulation_steps: int = 16,
        per_device_train_batch_size: int = 1,
        warmup_steps: int = 100,
        logging_steps: int = 1,
        use_fp16: bool = True,
        gradient_checkpointing: bool = True,
        overwrite_output_dir: bool = False,
        report_to: str = "none",
        optim: str = "adamw_torch",
        save_total_limit: Optional[int] = None
    ):
        def formatted_message(q, a):
            return f"<s>[INST] {q} [/INST]{a}</s>"
    
        def format_questions(subject, value, user_name, bot_name) -> list:
            other_names = ["Freddy", "Jack"]
            if user_name in other_names:
                other_names.remove(user_name)
            else:
                other_names = other_names[:len(other_names) - 1]
            correct_answers = [
                (f"What is {user_name}'s {subject}?", "I do not know"),
                (f"Incorrect. {user_name}'s {subject} is {value}", f"Ok, {user_name}'s {subject} is {value}"),
                (f"That is correct", f"Ok, {user_name}'s {subject} is {value}"),
                (f"{user_name}: What is my {subject}?", f"{bot_name}: {value}"),
                (f"{user_name}: Correct", f"{bot_name}: Ok, got it.")
            ]
            incorrect_answers = []
            for name in other_names:
                incorrect_answers.append((
                    f"What is {name}'s {subject}?",
                    value
                ))
                incorrect_answers.append((
                    f"That is incorrect.", 
                    "Then I do not know"
                ))
                incorrect_answers.append((
                    f"What is {name}'s {subject}?", 
                    "I do not know"
                ))
                incorrect_answers.append((
                    f"{name}: What is my {subject}?", 
                    f"{bot_name}: I do not know"
                ))
            return correct_answers + incorrect_answers + correct_answers
        
        formatted_questions = []
        for question in training_data:
            formatted_questions += format_questions(
                subject=question[0],
                value=question[1],
                user_name=username,
                bot_name=botname
            )

        dataset = Dataset.from_dict({"text": [
            formatted_message(
                question[0], 
                question[1]
            ) for question in formatted_questions
        ]})
        
        try:
            self._model = self._load_peft_model(self._model)
        except AttributeError as e:
            self.logger.error(f"Error applying PEFT configuration: {e}")

        # Get the latest step number from existing checkpoints
        last_step = 0
        if os.path.exists(self.finetuned_model_directory):
            checkpoints = [
                d for d in os.listdir(self.finetuned_model_directory)
                if d.startswith("checkpoint-")
            ]
            if checkpoints:
                last_step = max(
                    int(cp.split("-")[1]) 
                    for cp in checkpoints
                )

        # Define Training Arguments with resumed training
        training_args = TrainingArguments(
            output_dir=self.finetuned_model_directory,
            per_device_train_batch_size=per_device_train_batch_size,
            gradient_accumulation_steps=gradient_accumulation_steps,
            learning_rate=learning_rate,
            warmup_steps=warmup_steps,
            num_train_epochs=num_train_epochs,
            max_steps=last_step + training_steps,  # Increment the step count
            logging_steps=logging_steps,
            save_steps=save_steps,
            save_total_limit=save_total_limit,
            fp16=use_fp16,
            optim=optim,
            gradient_checkpointing=gradient_checkpointing,
            report_to=report_to,
            overwrite_output_dir=overwrite_output_dir
        )

        # Train the QLoRA Model on Conversations
        def tokenize_function(examples):
            tokens = self._tokenizer(examples["text"], truncation=True, padding="max_length", max_length=128)
            tokens["labels"] = tokens["input_ids"].copy()
            return tokens

        self._tokenizer.pad_token = self._tokenizer.eos_token
        tokenized_dataset = dataset.map(tokenize_function, batched=True)

        trainer = Trainer(
            model=self._model,
            args=training_args,
            train_dataset=tokenized_dataset
        )

        resume_checkpoint = None
        if last_step > 0 and self.latest_checkpoint:
            resume_checkpoint = self.latest_checkpoint
        
        self.logger.info(
            f"Resuming training from step {last_step}..."
            if resume_checkpoint
            else "No valid checkpoint found. Starting from scratch."
        )

        trainer.train(resume_from_checkpoint=resume_checkpoint)

        self.logger.info("Training completed.")
        self._save_finetuned_model()
    
    def _save_finetuned_model(self):
        self.logger.info("Saving finetuned model")
        self._model.save_pretrained(self.adapter_path)
        
        # Create minimal config
        minimal_config = {
            "name_or_path": self.model_path,
            "tokenizer_class": self._tokenizer.__class__.__name__,
            "model_max_length": self._tokenizer.model_max_length,
            "padding_side": self._tokenizer.padding_side,
            "truncation_side": getattr(self._tokenizer, "truncation_side", "right"),
            "special_tokens": {
                "bos_token": self._tokenizer.bos_token,
                "eos_token": self._tokenizer.eos_token,
                "unk_token": self._tokenizer.unk_token,
                "pad_token": self._tokenizer.pad_token,
            }
        }
        
        if hasattr(self._tokenizer, 'chat_template') and self._tokenizer.chat_template:
            minimal_config["chat_template"] = self._tokenizer.chat_template
                
        # Save the config
        config_path = os.path.join(self.adapter_path, "tokenizer_config.json")
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(minimal_config, f, indent=2, ensure_ascii=False)

        print(f"✅ QLoRA Adapter saved to: {self.adapter_path}")



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'llm_settings.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFrame,
    QGridLayout, QGroupBox, QHBoxLayout, QLabel,
    QLayout, QLineEdit, QPushButton, QScrollArea,
    QSizePolicy, QSpacerItem, QWidget)

from airunner.widgets.slider.slider_widget import SliderWidget

class Ui_llm_settings_widget(object):
    def setupUi(self, llm_settings_widget):
        if not llm_settings_widget.objectName():
            llm_settings_widget.setObjectName(u"llm_settings_widget")
        llm_settings_widget.resize(581, 742)
        self.gridLayout_4 = QGridLayout(llm_settings_widget)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.gridLayout_4.setHorizontalSpacing(0)
        self.gridLayout_4.setVerticalSpacing(10)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.label_2 = QLabel(llm_settings_widget)
        self.label_2.setObjectName(u"label_2")
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        self.label_2.setFont(font)

        self.gridLayout_4.addWidget(self.label_2, 0, 0, 1, 1)

        self.scrollArea = QScrollArea(llm_settings_widget)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setFrameShadow(QFrame.Shadow.Plain)
        self.scrollArea.setLineWidth(0)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 579, 700))
        self.gridLayout = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setHorizontalSpacing(0)
        self.gridLayout.setVerticalSpacing(10)
        self.gridLayout.setContentsMargins(0, 0, 10, 0)
        self.override_parameters = QGroupBox(self.scrollAreaWidgetContents)
        self.override_parameters.setObjectName(u"override_parameters")
        font1 = QFont()
        font1.setPointSize(8)
        self.override_parameters.setFont(font1)
        self.override_parameters.setCheckable(True)
        self.override_parameters.setChecked(True)
        self.gridLayout_12 = QGridLayout(self.override_parameters)
        self.gridLayout_12.setObjectName(u"gridLayout_12")
        self.gridLayout_12.setHorizontalSpacing(0)
        self.gridLayout_12.setVerticalSpacing(10)
        self.gridLayout_12.setContentsMargins(0, 0, 0, 0)
        self.widget_19 = QWidget(self.override_parameters)
        self.widget_19.setObjectName(u"widget_19")
        self.gridLayout_2 = QGridLayout(self.widget_19)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setHorizontalSpacing(0)
        self.gridLayout_2.setVerticalSpacing(10)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.seed = QLineEdit(self.widget_19)
        self.seed.setObjectName(u"seed")
        self.seed.setMinimumSize(QSize(0, 20))

        self.gridLayout_2.addWidget(self.seed, 0, 0, 1, 1)

        self.random_seed = QCheckBox(self.widget_19)
        self.random_seed.setObjectName(u"random_seed")
        self.random_seed.setMinimumSize(QSize(0, 20))

        self.gridLayout_2.addWidget(self.random_seed, 0, 1, 1, 1)


        self.gridLayout_12.addWidget(self.widget_19, 6, 0, 1, 1)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setSpacing(10)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.length_penalty = SliderWidget(self.override_parameters)
        self.length_penalty.setObjectName(u"length_penalty")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.length_penalty.sizePolicy().hasHeightForWidth())
        self.length_penalty.setSizePolicy(sizePolicy)
        self.length_penalty.setMinimumSize(QSize(0, 0))
        self.length_penalty.setProperty("slider_minimum", -100)
        self.length_penalty.setProperty("slider_maximum", 1000)
        self.length_penalty.setProperty("spinbox_minimum", 0.000000000000000)
        self.length_penalty.setProperty("spinbox_maximum", 1.000000000000000)
        self.length_penalty.setProperty("display_as_float", True)
        self.length_penalty.setProperty("slider_single_step", 1)
        self.length_penalty.setProperty("slider_page_step", 10)
        self.length_penalty.setProperty("spinbox_single_step", 0.010000000000000)
        self.length_penalty.setProperty("spinbox_page_step", 0.100000000000000)

        self.horizontalLayout_3.addWidget(self.length_penalty)

        self.num_beams = SliderWidget(self.override_parameters)
        self.num_beams.setObjectName(u"num_beams")
        sizePolicy.setHeightForWidth(self.num_beams.sizePolicy().hasHeightForWidth())
        self.num_beams.setSizePolicy(sizePolicy)
        self.num_beams.setMinimumSize(QSize(0, 0))
        self.num_beams.setProperty("slider_minimum", 1)
        self.num_beams.setProperty("slider_maximum", 100)
        self.num_beams.setProperty("spinbox_minimum", 0.000000000000000)
        self.num_beams.setProperty("spinbox_maximum", 100.000000000000000)
        self.num_beams.setProperty("display_as_float", False)
        self.num_beams.setProperty("slider_single_step", 1)
        self.num_beams.setProperty("slider_page_step", 10)
        self.num_beams.setProperty("spinbox_single_step", 0.010000000000000)
        self.num_beams.setProperty("spinbox_page_step", 0.100000000000000)

        self.horizontalLayout_3.addWidget(self.num_beams)


        self.gridLayout_12.addLayout(self.horizontalLayout_3, 3, 0, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(10)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setSizeConstraint(QLayout.SizeConstraint.SetDefaultConstraint)
        self.top_p = SliderWidget(self.override_parameters)
        self.top_p.setObjectName(u"top_p")
        sizePolicy.setHeightForWidth(self.top_p.sizePolicy().hasHeightForWidth())
        self.top_p.setSizePolicy(sizePolicy)
        self.top_p.setMinimumSize(QSize(0, 0))
        self.top_p.setProperty("slider_minimum", 1)
        self.top_p.setProperty("slider_maximum", 1000)
        self.top_p.setProperty("spinbox_minimum", 0.000000000000000)
        self.top_p.setProperty("spinbox_maximum", 1.000000000000000)
        self.top_p.setProperty("display_as_float", True)
        self.top_p.setProperty("slider_single_step", 1)
        self.top_p.setProperty("slider_page_step", 10)
        self.top_p.setProperty("spinbox_single_step", 0.010000000000000)
        self.top_p.setProperty("spinbox_page_step", 0.100000000000000)

        self.horizontalLayout.addWidget(self.top_p)

        self.max_new_tokens = SliderWidget(self.override_parameters)
        self.max_new_tokens.setObjectName(u"max_new_tokens")
        sizePolicy.setHeightForWidth(self.max_new_tokens.sizePolicy().hasHeightForWidth())
        self.max_new_tokens.setSizePolicy(sizePolicy)
        self.max_new_tokens.setMinimumSize(QSize(0, 0))
        self.max_new_tokens.setProperty("slider_minimum", 1)
        self.max_new_tokens.setProperty("slider_maximum", 2556)
        self.max_new_tokens.setProperty("spinbox_minimum", 1.000000000000000)
        self.max_new_tokens.setProperty("spinbox_maximum", 2556.000000000000000)
        self.max_new_tokens.setProperty("display_as_float", False)
        self.max_new_tokens.setProperty("slider_single_step", 1)
        self.max_new_tokens.setProperty("slider_page_step", 2556)
        self.max_new_tokens.setProperty("spinbox_single_step", 1)
        self.max_new_tokens.setProperty("spinbox_page_step", 2556)

        self.horizontalLayout.addWidget(self.max_new_tokens)


        self.gridLayout_12.addLayout(self.horizontalLayout, 0, 0, 1, 1)

        self.line = QFrame(self.override_parameters)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_12.addWidget(self.line, 9, 0, 1, 1)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setSpacing(10)
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.sequences = SliderWidget(self.override_parameters)
        self.sequences.setObjectName(u"sequences")
        sizePolicy.setHeightForWidth(self.sequences.sizePolicy().hasHeightForWidth())
        self.sequences.setSizePolicy(sizePolicy)
        self.sequences.setMinimumSize(QSize(0, 0))
        self.sequences.setProperty("slider_minimum", 1)
        self.sequences.setProperty("slider_maximum", 100)
        self.sequences.setProperty("spinbox_minimum", 0.000000000000000)
        self.sequences.setProperty("spinbox_maximum", 100.000000000000000)
        self.sequences.setProperty("display_as_float", False)
        self.sequences.setProperty("slider_single_step", 1)
        self.sequences.setProperty("slider_page_step", 10)
        self.sequences.setProperty("spinbox_single_step", 0.010000000000000)
        self.sequences.setProperty("spinbox_page_step", 0.100000000000000)

        self.horizontalLayout_6.addWidget(self.sequences)

        self.top_k = SliderWidget(self.override_parameters)
        self.top_k.setObjectName(u"top_k")
        sizePolicy.setHeightForWidth(self.top_k.sizePolicy().hasHeightForWidth())
        self.top_k.setSizePolicy(sizePolicy)
        self.top_k.setMinimumSize(QSize(0, 0))
        self.top_k.setProperty("slider_minimum", 0)
        self.top_k.setProperty("slider_maximum", 256)
        self.top_k.setProperty("spinbox_minimum", 0.000000000000000)
        self.top_k.setProperty("spinbox_maximum", 256.000000000000000)
        self.top_k.setProperty("display_as_float", False)
        self.top_k.setProperty("slider_single_step", 1)
        self.top_k.setProperty("slider_page_step", 10)
        self.top_k.setProperty("spinbox_single_step", 1)
        self.top_k.setProperty("spinbox_page_step", 10)

        self.horizontalLayout_6.addWidget(self.top_k)


        self.gridLayout_12.addLayout(self.horizontalLayout_6, 5, 0, 1, 1)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setSpacing(10)
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.ngram_size = SliderWidget(self.override_parameters)
        self.ngram_size.setObjectName(u"ngram_size")
        sizePolicy.setHeightForWidth(self.ngram_size.sizePolicy().hasHeightForWidth())
        self.ngram_size.setSizePolicy(sizePolicy)
        self.ngram_size.setMinimumSize(QSize(0, 0))
        self.ngram_size.setProperty("slider_minimum", 0)
        self.ngram_size.setProperty("slider_maximum", 20)
        self.ngram_size.setProperty("spinbox_minimum", 0.000000000000000)
        self.ngram_size.setProperty("spinbox_maximum", 20.000000000000000)
        self.ngram_size.setProperty("display_as_float", False)
        self.ngram_size.setProperty("slider_single_step", 1)
        self.ngram_size.setProperty("slider_page_step", 1)
        self.ngram_size.setProperty("spinbox_single_step", 1.000000000000000)
        self.ngram_size.setProperty("spinbox_page_step", 1.000000000000000)

        self.horizontalLayout_5.addWidget(self.ngram_size)

        self.temperature = SliderWidget(self.override_parameters)
        self.temperature.setObjectName(u"temperature")
        sizePolicy.setHeightForWidth(self.temperature.sizePolicy().hasHeightForWidth())
        self.temperature.setSizePolicy(sizePolicy)
        self.temperature.setMinimumSize(QSize(0, 0))
        self.temperature.setProperty("slider_minimum", 1)
        self.temperature.setProperty("slider_maximum", 10000)
        self.temperature.setProperty("spinbox_minimum", 0.000100000000000)
        self.temperature.setProperty("spinbox_maximum", 1.000000000000000)
        self.temperature.setProperty("display_as_float", True)
        self.temperature.setProperty("slider_single_step", 1)
        self.temperature.setProperty("slider_page_step", 10)
        self.temperature.setProperty("spinbox_single_step", 0.010000000000000)
        self.temperature.setProperty("spinbox_page_step", 0.100000000000000)

        self.horizontalLayout_5.addWidget(self.temperature)


        self.gridLayout_12.addLayout(self.horizontalLayout_5, 4, 0, 1, 1)

        self.gridLayout_3 = QGridLayout()
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setHorizontalSpacing(0)
        self.gridLayout_3.setVerticalSpacing(10)
        self.early_stopping = QCheckBox(self.override_parameters)
        self.early_stopping.setObjectName(u"early_stopping")

        self.gridLayout_3.addWidget(self.early_stopping, 0, 0, 1, 1)

        self.do_sample = QCheckBox(self.override_parameters)
        self.do_sample.setObjectName(u"do_sample")

        self.gridLayout_3.addWidget(self.do_sample, 0, 1, 1, 1)

        self.use_cache = QCheckBox(self.override_parameters)
        self.use_cache.setObjectName(u"use_cache")

        self.gridLayout_3.addWidget(self.use_cache, 0, 2, 1, 1)


        self.gridLayout_12.addLayout(self.gridLayout_3, 8, 0, 1, 1)

        self.pushButton = QPushButton(self.override_parameters)
        self.pushButton.setObjectName(u"pushButton")

        self.gridLayout_12.addWidget(self.pushButton, 11, 0, 1, 1)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(10)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.repetition_penalty = SliderWidget(self.override_parameters)
        self.repetition_penalty.setObjectName(u"repetition_penalty")
        sizePolicy.setHeightForWidth(self.repetition_penalty.sizePolicy().hasHeightForWidth())
        self.repetition_penalty.setSizePolicy(sizePolicy)
        self.repetition_penalty.setMinimumSize(QSize(0, 0))
        self.repetition_penalty.setProperty("slider_minimum", 1)
        self.repetition_penalty.setProperty("slider_maximum", 10000)
        self.repetition_penalty.setProperty("spinbox_minimum", 0.010000000000000)
        self.repetition_penalty.setProperty("spinbox_maximum", 100.000000000000000)
        self.repetition_penalty.setProperty("display_as_float", True)
        self.repetition_penalty.setProperty("slider_single_step", 0)
        self.repetition_penalty.setProperty("slider_page_step", 1)
        self.repetition_penalty.setProperty("spinbox_single_step", 1.000000000000000)
        self.repetition_penalty.setProperty("spinbox_page_step", 10.000000000000000)

        self.horizontalLayout_2.addWidget(self.repetition_penalty)

        self.min_length = SliderWidget(self.override_parameters)
        self.min_length.setObjectName(u"min_length")
        sizePolicy.setHeightForWidth(self.min_length.sizePolicy().hasHeightForWidth())
        self.min_length.setSizePolicy(sizePolicy)
        self.min_length.setMinimumSize(QSize(0, 0))
        self.min_length.setProperty("slider_minimum", 1)
        self.min_length.setProperty("slider_maximum", 2556)
        self.min_length.setProperty("spinbox_minimum", 1.000000000000000)
        self.min_length.setProperty("spinbox_maximum", 2556.000000000000000)
        self.min_length.setProperty("display_as_float", False)
        self.min_length.setProperty("slider_single_step", 1)
        self.min_length.setProperty("slider_page_step", 2556)
        self.min_length.setProperty("spinbox_single_step", 1)
        self.min_length.setProperty("spinbox_page_step", 2556)

        self.horizontalLayout_2.addWidget(self.min_length)


        self.gridLayout_12.addLayout(self.horizontalLayout_2, 2, 0, 1, 1)


        self.gridLayout.addWidget(self.override_parameters, 3, 0, 1, 1)

        self.verticalSpacer = QSpacerItem(20, 263, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 4, 0, 1, 1)

        self.model_version_container = QWidget(self.scrollAreaWidgetContents)
        self.model_version_container.setObjectName(u"model_version_container")
        self.gridLayout_11 = QGridLayout(self.model_version_container)
        self.gridLayout_11.setObjectName(u"gridLayout_11")
        self.gridLayout_11.setHorizontalSpacing(0)
        self.gridLayout_11.setVerticalSpacing(10)
        self.gridLayout_11.setContentsMargins(0, 0, 0, 0)
        self.model_version = QComboBox(self.model_version_container)
        self.model_version.setObjectName(u"model_version")

        self.gridLayout_11.addWidget(self.model_version, 1, 0, 1, 1)

        self.model_version_label = QLabel(self.model_version_container)
        self.model_version_label.setObjectName(u"model_version_label")
        self.model_version_label.setFont(font1)

        self.gridLayout_11.addWidget(self.model_version_label, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.model_version_container, 1, 0, 1, 1)

        self.prompt_template_container = QWidget(self.scrollAreaWidgetContents)
        self.prompt_template_container.setObjectName(u"prompt_template_container")
        self.gridLayout_17 = QGridLayout(self.prompt_template_container)
        self.gridLayout_17.setObjectName(u"gridLayout_17")
        self.gridLayout_17.setHorizontalSpacing(0)
        self.gridLayout_17.setVerticalSpacing(10)
        self.gridLayout_17.setContentsMargins(0, 0, 0, 0)
        self.prompt_template = QComboBox(self.prompt_template_container)
        self.prompt_template.setObjectName(u"prompt_template")

        self.gridLayout_17.addWidget(self.prompt_template, 1, 0, 1, 1)

        self.prompt_template_label = QLabel(self.prompt_template_container)
        self.prompt_template_label.setObjectName(u"prompt_template_label")
        self.prompt_template_label.setFont(font1)

        self.gridLayout_17.addWidget(self.prompt_template_label, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.prompt_template_container, 2, 0, 1, 1)

        self.model_type_container = QWidget(self.scrollAreaWidgetContents)
        self.model_type_container.setObjectName(u"model_type_container")
        self.model_type_container.setFont(font1)
        self.gridLayout_10 = QGridLayout(self.model_type_container)
        self.gridLayout_10.setObjectName(u"gridLayout_10")
        self.gridLayout_10.setHorizontalSpacing(0)
        self.gridLayout_10.setVerticalSpacing(10)
        self.gridLayout_10.setContentsMargins(0, 0, 0, 0)
        self.model = QComboBox(self.model_type_container)
        self.model.setObjectName(u"model")

        self.gridLayout_10.addWidget(self.model, 1, 0, 1, 1)

        self.model_type_label = QLabel(self.model_type_container)
        self.model_type_label.setObjectName(u"model_type_label")

        self.gridLayout_10.addWidget(self.model_type_label, 0, 0, 1, 1)


        self.gridLayout.addWidget(self.model_type_container, 0, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout_4.addWidget(self.scrollArea, 2, 0, 1, 1)

        self.line_2 = QFrame(llm_settings_widget)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setFrameShape(QFrame.Shape.HLine)
        self.line_2.setFrameShadow(QFrame.Shadow.Sunken)

        self.gridLayout_4.addWidget(self.line_2, 1, 0, 1, 1)

        QWidget.setTabOrder(self.model, self.model_version)
        QWidget.setTabOrder(self.model_version, self.seed)
        QWidget.setTabOrder(self.seed, self.random_seed)

        self.retranslateUi(llm_settings_widget)
        self.override_parameters.toggled.connect(llm_settings_widget.override_parameters_toggled)
        self.early_stopping.toggled.connect(llm_settings_widget.early_stopping_toggled)
        self.pushButton.clicked.connect(llm_settings_widget.reset_settings_to_default_clicked)
        self.do_sample.toggled.connect(llm_settings_widget.do_sample_toggled)
        self.use_cache.clicked["bool"].connect(llm_settings_widget.toggle_use_cache)

        self.model_version.setCurrentIndex(-1)


        QMetaObject.connectSlotsByName(llm_settings_widget)
    # setupUi

    def retranslateUi(self, llm_settings_widget):
        llm_settings_widget.setWindowTitle(QCoreApplication.translate("llm_settings_widget", u"Form", None))
        self.label_2.setText(QCoreApplication.translate("llm_settings_widget", u"LLM Settings", None))
        self.override_parameters.setTitle(QCoreApplication.translate("llm_settings_widget", u"Override Prameters", None))
        self.random_seed.setText(QCoreApplication.translate("llm_settings_widget", u"Random seed", None))
        self.length_penalty.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.length_penalty", None))
        self.length_penalty.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Length Penalty", None))
        self.num_beams.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.num_beams", None))
        self.num_beams.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Num Beams", None))
        self.top_p.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.top_p", None))
        self.top_p.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Top P", None))
        self.max_new_tokens.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.max_new_tokens", None))
        self.max_new_tokens.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Max New Tokens", None))
        self.sequences.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.sequences", None))
        self.sequences.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Sequences", None))
        self.top_k.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.top_k", None))
        self.top_k.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Top K", None))
        self.ngram_size.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.ngram_size", None))
        self.ngram_size.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Ngram Size", None))
        self.temperature.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.temperature", None))
        self.temperature.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Temperature", None))
        self.early_stopping.setText(QCoreApplication.translate("llm_settings_widget", u"Early stopping", None))
        self.do_sample.setText(QCoreApplication.translate("llm_settings_widget", u"Do sample", None))
        self.use_cache.setText(QCoreApplication.translate("llm_settings_widget", u"Use Cache", None))
        self.pushButton.setText(QCoreApplication.translate("llm_settings_widget", u"Reset Settings to Default", None))
        self.repetition_penalty.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.repetition_penalty", None))
        self.repetition_penalty.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Repetition Penalty", None))
        self.min_length.setProperty("settings_property", QCoreApplication.translate("llm_settings_widget", u"llm_generator_settings.min_length", None))
        self.min_length.setProperty("label_text", QCoreApplication.translate("llm_settings_widget", u"Min Length", None))
        self.model_version_label.setText(QCoreApplication.translate("llm_settings_widget", u"Model Version", None))
        self.prompt_template_label.setText(QCoreApplication.translate("llm_settings_widget", u"Prompt Template", None))
        self.model_type_label.setText(QCoreApplication.translate("llm_settings_widget", u"Model Type", None))
    # retranslateUi




from airunner.data.models.active_grid_settings import ActiveGridSettings
from airunner.data.models.application_settings import ApplicationSettings
from airunner.data.models.controlnet_settings import ControlnetSettings
from airunner.data.models.image_to_image_settings import ImageToImageSettings
from airunner.data.models.outpaint_settings import OutpaintSettings
from airunner.data.models.chatstore import Chatstore
from airunner.data.models.drawingpad_settings import DrawingPadSettings
from airunner.data.models.metadata_settings import MetadataSettings
from airunner.data.models.generator_settings import GeneratorSettings
from airunner.data.models.llm_generator_settings import LLMGeneratorSettings
from airunner.data.models.tts_settings import TTSSettings
from airunner.data.models.speech_t5_settings import SpeechT5Settings
from airunner.data.models.espeak_settings import EspeakSettings
from airunner.data.models.stt_settings import STTSettings
from airunner.data.models.schedulers import Schedulers
from airunner.data.models.brush_settings import BrushSettings
from airunner.data.models.grid_settings import GridSettings
from airunner.data.models.path_settings import PathSettings
from airunner.data.models.memory_settings import MemorySettings
from airunner.data.models.chatbot import Chatbot
from airunner.data.models.user import User
from airunner.data.models.target_files import TargetFiles
from airunner.data.models.target_directories import TargetDirectories
from airunner.data.models.ai_models import AIModels
from airunner.data.models.shortcut_keys import ShortcutKeys
from airunner.data.models.lora import Lora
from airunner.data.models.saved_prompt import SavedPrompt
from airunner.data.models.embedding import Embedding
from airunner.data.models.prompt_template import PromptTemplate
from airunner.data.models.controlnet_model import ControlnetModel
from airunner.data.models.font_setting import FontSetting
from airunner.data.models.pipeline_model import PipelineModel
from airunner.data.models.conversation import Conversation
from airunner.data.models.summary import Summary
from airunner.data.models.image_filter import ImageFilter
from airunner.data.models.image_filter_value import ImageFilterValue
from airunner.data.models.whisper_settings import WhisperSettings
from airunner.data.models.news import RSSFeed, Category, Article
from airunner.data.models.tab import Tab
from airunner.data.models.base import Base

__all__ = [
    'ActiveGridSettings',
    'ApplicationSettings',
    'ControlnetSettings',
    'ImageToImageSettings',
    'OutpaintSettings',
    'Chatstore',
    'DrawingPadSettings',
    'MetadataSettings',
    'GeneratorSettings',
    'LLMGeneratorSettings',
    'TTSSettings',
    'SpeechT5Settings',
    'EspeakSettings',
    'STTSettings',
    'Schedulers',
    'BrushSettings',
    'GridSettings',
    'PathSettings',
    'MemorySettings',
    'Chatbot',
    'User',
    'TargetFiles',
    'TargetDirectories',
    'AIModels',
    'ShortcutKeys',
    'Lora',
    'SavedPrompt',
    'Embedding',
    'PromptTemplate',
    'ControlnetModel',
    'FontSetting',
    'PipelineModel',
    'Conversation',
    'Summary',
    'ImageFilter',
    'ImageFilterValue',
    'WhisperSettings',
    'RSSFeed',
    'Category',
    'Article',
    'Tab',
    'Base',
]



from airunner.workers.agent_worker import AgentWorker
from airunner.workers.audio_capture_worker import AudioCaptureWorker
from airunner.workers.audio_processor_worker import AudioProcessorWorker
from airunner.workers.directory_watcher import DirectoryWatcher
from airunner.workers.llm_generate_worker import LLMGenerateWorker
from airunner.workers.llm_response_worker import LLMResponseWorker
from airunner.workers.mask_generator_worker import MaskGeneratorWorker
from airunner.workers.model_scanner_worker import ModelScannerWorker
from airunner.workers.sd_worker import SDWorker
from airunner.workers.tts_generator_worker import TTSGeneratorWorker
from airunner.workers.tts_vocalizer_worker import TTSVocalizerWorker
from airunner.workers.watch_state_worker import WatchStateWorker
from airunner.workers.worker import Worker


__all__ = [
    "AgentWorker",
    "AudioCaptureWorker",
    "AudioProcessorWorker",
    "DirectoryWatcher",
    "LLMGenerateWorker",
    "LLMResponseWorker",
    "MaskGeneratorWorker",
    "ModelScannerWorker",
    "SDWorker",
    "TTSGeneratorWorker",
    "TTSVocalizerWorker",
    "WatchStateWorker",
    "Worker",
]


from typing import Type, Optional
from abc import ABCMeta
import os
import torch
import enum
import librosa

from openvoice.mel_processing import spectrogram_torch
from openvoice import se_extractor
from openvoice.api import OpenVoiceBaseClass, ToneColorConverter
from melo.api import TTS

from airunner.settings import AIRUNNER_TTS_SPEAKER_RECORDING_PATH
from airunner.enums import SignalCode
from airunner.handlers.tts.tts_handler import TTSHandler
from airunner.enums import ModelType, ModelStatus


class AvailableLanguage(enum.Enum):
    EN_NEWEST = "EN_NEWEST"
    EN = "EN"
    ES = "ES"
    FR = "FR"
    ZH = "ZH"
    JP = "JP"
    KR = "KR"


class StreamingToneColorConverter(ToneColorConverter):
    def __init__(self, *args, **kwargs):
        OpenVoiceBaseClass.__init__(self, *args, **kwargs)
        self.version = getattr(self.hps, '_version_', "v1")

    def convert(
        self, 
        audio_src_path, 
        src_se, 
        tgt_se, 
        output_path=None, 
        tau=0.3, 
        message="default"
    ):
        hps = self.hps
        # load audio
        try:
            audio, sample_rate = librosa.load(
                audio_src_path, 
                sr=hps.data.sampling_rate
            )
        except ValueError as e:
            print(f"Error: {e}")
            return None
        
        audio = torch.tensor(audio).float()
        
        with torch.no_grad():
            y = torch.FloatTensor(audio).to(self.device)
            y = y.unsqueeze(0)
            spec = spectrogram_torch(
                y, 
                hps.data.filter_length,
                hps.data.sampling_rate,
                hps.data.hop_length, 
                hps.data.win_length,
                center=False
            ).to(
                self.device
            )
            spec_lengths = torch.LongTensor(
                [spec.size(-1)]
            ).to(
                self.device
            )
            audio = self.model.voice_conversion(
                spec, 
                spec_lengths, 
                sid_src=src_se, 
                sid_tgt=tgt_se, 
                tau=tau
            )[0][0, 0].data.cpu().float().numpy()
            return audio


class OpenVoiceHandler(TTSHandler, metaclass=ABCMeta):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        speaker_recording_path = os.path.expanduser(
            AIRUNNER_TTS_SPEAKER_RECORDING_PATH
        )
        self._checkpoint_converter_path: str = os.path.join(
            self.path_settings.tts_model_path,
            'openvoice/checkpoints_v2/converter'
        )
        self._output_dir: str = os.path.join(
            self.path_settings.tts_model_path,
            'openvoice/outputs_v2'
        )
        self._tone_color_converter: Optional[Type[ToneColorConverter]] = None
        self.model: Optional[TTS] = None
        self.src_path: str = f'{self._output_dir}/tmp.wav'
        self._speed: float = 1.0
        self._language: AvailableLanguage = AvailableLanguage.EN_NEWEST
        self._reference_speaker = os.path.expanduser(speaker_recording_path)
        self._target_se, self._audio_name = se_extractor.get_se(
            self._reference_speaker, 
            self.tone_color_converter, 
            vad=True
        )

    @property
    def device(self):
        use_cuda = torch.cuda.is_available()
        card_index = 0
        return f"cuda:{card_index}" if use_cuda else "cpu"
    
    @property
    def tone_color_converter(self) -> StreamingToneColorConverter:
        if not self._tone_color_converter:
            self._tone_color_converter = StreamingToneColorConverter(
                f'{self._checkpoint_converter_path}/config.json',
                device=self.device
            )
            self._tone_color_converter.load_ckpt(
                f'{self._checkpoint_converter_path}/checkpoint.pth'
            )
        return self._tone_color_converter

    def generate(self, message: str):
        speaker_ids = self.model.hps.data.spk2id

        for speaker_key in speaker_ids.keys():
            speaker_id = speaker_ids[speaker_key]
            speaker_key = speaker_key.lower().replace('_', '-')
            
            source_se = torch.load(
                os.path.join(
                    self.path_settings.tts_model_path,
                    f'openvoice/checkpoints_v2/base_speakers/ses/{speaker_key}.pth', 
                ),
                map_location=self.device
            )
            
            self.model.tts_to_file(
                message, 
                speaker_id, 
                self.src_path, 
                speed=self._speed
            )

            # Run the tone color converter
            output_path = os.path.join(
                self.path_settings.tts_model_path,
                f'openvoice/{self._output_dir}/output_v2_{speaker_key}.wav'
            )

            encode_message = "@MyShell"
            
            response = self.tone_color_converter.convert(
                audio_src_path=self.src_path, 
                src_se=source_se, 
                tgt_se=self._target_se, 
                output_path=output_path,
                message=encode_message
            )

            if response is not None:
                self.emit_signal(
                    SignalCode.TTS_GENERATOR_WORKER_ADD_TO_STREAM_SIGNAL, 
                    {
                        "message": response
                    }
                )

    def load(self, _target_model=None):
        self.logger.debug("Initializing OpenVoice")
        self.unload()
        self.change_model_status(ModelType.TTS, ModelStatus.LOADING)
        self._initialize()
        self.model = TTS(
            language=self._language.value,
            device=self.device
        )
        self.change_model_status(ModelType.TTS, ModelStatus.LOADED)

    def unload(self):
        self.logger.debug("Unloading OpenVoice")
        self.change_model_status(ModelType.TTS, ModelStatus.LOADING)
        self.model = None
        self.change_model_status(ModelType.TTS, ModelStatus.UNLOADED)

    def unblock_tts_generator_signal(self):
        pass

    def interrupt_process_signal(self):
        pass

    def _initialize(self):
        vad, vad_utils = torch.hub.load(
            repo_or_dir='snakers4/silero-vad',
            model='silero_vad',
            force_reload=False,
            onnx=False
        )
        os.makedirs(self._output_dir, exist_ok=True)


from sqlalchemy import (
    Column, 
    Integer, 
    Boolean, 
    String, 
    Float, 
    JSON, 
    ForeignKey, 
    BigInteger
)
from sqlalchemy.orm import relationship

from airunner.data.models.base import BaseModel
from airunner.settings import AIRUNNER_SD_DEFAULT_VAE_PATH, AIRUNNER_DEFAULT_SCHEDULER


class GeneratorSettings(BaseModel):
    __tablename__ = 'generator_settings'
    id = Column(Integer, primary_key=True, autoincrement=True)
    pipeline_action = Column(String, default="txt2img")
    generator_name = Column(String, default="stablediffusion")
    quality_effects = Column(String, default="")
    image_preset = Column(String, default="")
    prompt = Column(String, default="")
    negative_prompt = Column(String, default="")
    second_prompt = Column(String, default="")
    second_negative_prompt = Column(String, default="")
    random_seed = Column(Boolean, default=True)
    model_name = Column(String, default="")
    model = Column(Integer, ForeignKey('aimodels.id'), nullable=True)
    aimodel = relationship("AIModels", back_populates="generator_settings")
    vae = Column(String, default=AIRUNNER_SD_DEFAULT_VAE_PATH)
    scheduler = Column(String, default=AIRUNNER_DEFAULT_SCHEDULER)
    variation = Column(Boolean, default=False)
    use_prompt_builder = Column(Boolean, default=False)
    version = Column(String, default="SD 1.5")
    is_preset = Column(Boolean, default=False)
    use_compel = Column(Boolean, default=True)

    steps = Column(Integer, default=20)
    ddim_eta = Column(Float, default=0.5)
    scale = Column(Integer, default=750)
    seed = Column(BigInteger, default=42)
    prompt_triggers = Column(String, default="")
    strength = Column(Integer, default=50)
    n_samples = Column(Integer, default=1)
    clip_skip = Column(Integer, default=0)
    crops_coord_top_left = Column(JSON, default={"x": 0, "y": 0})
    original_size = Column(JSON, default={"width": 512, "height": 512})
    target_size = Column(JSON, default={"width": 1024, "height": 1024})
    negative_original_size = Column(JSON, default={"width": 512, "height": 512})
    negative_target_size = Column(JSON, default={"width": 512, "height": 512})

    lora_scale = Column(Integer, default=100)
